#!/usr/bin/python
# -*- coding: utf-8 -*-

USAGE = """
  usage: %prog [options]
  -s, --settings=file: settings file eg: jobs.settings
  -d, --dryrun: don't really touch the db
  -v, --verbose: can't you immagine?
"""

import re
import os
import csv
import sys
import email
import string
import random
import logging
import logging.handlers
from datetime import datetime

from django.utils.translation import ugettext as _
from django.contrib.auth.models import User
from django.utils import encoding
from django.db import transaction
from django.conf import settings
from django.core.files.base import ContentFile
from django.core.mail.message import EmailMessage
from django.contrib.auth.models import Group, User

from jmb.fax.models import Fax, FaxNumber
from jmb.core.utils import optionparse

opt, args = optionparse.parse(USAGE)
os.environ['DJANGO_SETTINGS_MODULE'] = opt.settings



class ParseMail:
    def __init__(self, mail, logging):
        self.log = logging
        self.msg = email.message_from_string(mail)
        self.d_dati = {}
        self.verbose = opt.verbose
        self.clean_from_address = ''
        self.fax_number = None

    def parse_header(self):
        received = self.msg['Received']
        to = self.msg['To']
        fromm = self.msg['From']
        subject = self.msg['Subject']
        docuser = None

        if self.verbose:
            self.log.debug("received : %s" % (received,))
            self.log.debug("to : %s" % (to,))
            self.log.debug("from : %s" % (fromm,))
            self.log.debug("subject : %s" % (subject,))

        # Pulisco il from della mail per fare il controllo sull'utenza
        if fromm.find('<') > 0:
            ff = re.compile('^.*\<(?P<from>[a-zA-Z0-9_.@-]*)\>$', re.M).match(fromm)
        else:
            ff = re.compile('^(?P<from>[a-zA-Z0-9_.@-]*)$', re.M).match(fromm)

        if ff:
            self.clean_from_address = ff.group("from")
            self.log.debug("clean from addr : %s" % (self.clean_from_address,))

        # Controllo che l'indirizzo a cui viene spedita la mail sia faxdjango@dominio.xxx
        to_addr = None
        t = re.compile('^\"?\'?\<?(?P<to>[a-zA-Z0-9_.@-]*)\>?\'?\"?.*$', re.M).match(to)

        if t:
            to_addr = t.group("to")
            if self.verbose:
                self.log.debug("to addr : %s" % (to_addr,))

            indirizzo_to = to_addr[:to_addr.find('@')]
            if indirizzo_to != settings.FAX_VIA_MAIL_TO_ADDRESS:
                self.log.debug("Il to della mail non e' corretto. Dovrebbe essere : %s" % (indirizzo_to,))
                sys.exit(1)

        # Recupero il numero di fax dall'oggetto della mail e controllo che sia stato inserito un numero
        self.fax_number = subject
        if self.fax_number and not self.fax_number.isdigit():
            self.log.debug("Il numero di fax inserito non e' numerico.")
            sys.exit(1)


    @transaction.commit_manually
    def save_and_send_fax(self):
        lista = self.msg.get_payload()

        try:
            for part in lista:
                #self.log.debug("PART : %s" % part)
                maintype = part.get_content_maintype()

                # cerco la parte contenente l/gli attachement
                #
                if ( maintype != "text" and
                     maintype != "multipart" and
                     maintype != "message"):

                    file_content = ContentFile(part.get_payload(decode=1))
                    self.invia_fax(file_content)
        except Exception as e:
            self.log.error(e)
            transaction.rollback()
            raise Exception(e)
        else:
            transaction.commit()

    def invia_fax(self, file_content):
        from jumbo.middleware.thread_local import _thread_locals
        from django.contrib.auth.models import User

        # Setto nei thread_locals lo user, andando a cercare tra i DocumentUser
        # l'utente che l'indirizzo email del from
        try:
            fax_user = User.objects.get(email=self.clean_from_address)
            _thread_locals.user = fax_user
        except:
            raise Exception("Non esiste uno User con questo indirizzo Email")

        # Recupero uno dei device disponibili per l'utente loggato
        fax_numbers = FaxNumber.objects.all()
        if fax_numbers:
            device = fax_numbers[0].device.all()[0].name
            self.log.debug("DEVICE UTENTE %s" % device)
        else:
            self.log.debug("L'utente non ha i permessi per poter manadare i fax da nessun numero di telefono")
            sys.exit(1)

        f = Fax()
        f.subject = settings.FAX_VIA_MAIL_DEFAULT_SUBJECT_FAX
        f.priority = 120
        f.keywords = "fax mail"
        f.recipient = ''
        f.read = 1
        f.status = 3
        f.recipient_number = self.fax_number
        f.submitter = fax_user
        f.device = device
        f.sent_by = 'M'

        rdm_string = ''.join(random.sample(string.digits+string.letters, 5))
        nfilename = "fax_%s_%s.pdf" % (datetime.today().strftime("%Y%m%d_%H%M%S"), rdm_string)
        if opt.dryrun:
            self.log.debug("SALVO FILE %s" % nfilename)
        else:
            f.file.save(nfilename, file_content)

        if opt.dryrun:
            self.log.debug("SALVO E INVIO FAX AL NUMERO %s" % self.fax_number)
        else:
            f.save()
            f.send_fax()

        if settings.FAX_VIA_MAIL_NOTIFY_TO_USER:
            if opt.dryrun:
                self.log.debug("INVIO MAIL DI NOTIFICA A %s" % fax_user.email)
            else:
                email_message = EmailMessage(_("[FAX MAIL] - You've sent a fax via mail to : %s") % self.fax_number, '',
                                settings.FAX_FROM, [fax_user.email])
                email_message.attach_file(f.file.path)
                email_message.send()

        if opt.verbose:
            self.log.debug("Fine Invio Fax Via Mail")


if __name__ == '__main__':
    # Set up a specific logger with our desired output level
    jlogger = logging.getLogger('JumboDocLogger')
    jlogger.setLevel(logging.DEBUG)

    # Add the log message handler to the logger
    handler = logging.handlers.RotatingFileHandler(
        settings.FAX_LOG_FILENAME, maxBytes=4194304, backupCount=5)

    # create formatter
    formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
    # add formatter to handler
    handler.setFormatter(formatter)
    jlogger.addHandler(handler)

    mail = sys.stdin.read()
    if opt.verbose:
        jlogger.debug("------ INIZIO INVIO FAX VIA MAIL -------")
    try:
        p = ParseMail(mail, jlogger)
        p.parse_header()
        p.save_and_send_fax()
    except Exception as e:
        raise Exception(e)
