# coding: utf-8
"""
Servizio CodemaOnline
=====================

Modulo per spedizione e ricezione di fax dal servizio codemaonline che
neccessita:

* una casella pop3 cui invia la notifica di un fax in arrivo o la redirezione ad un trasport
  locale 

* una spedizione di un allegato pdf (altri formati ammessi) attraverso un
  suo servizio. 


Alternativa Cron o transport
=============================

Il modulo fornisce il comando :ref:`retrieve_codema_fax
<retrieve-codema-fax>` che analizza le mail inviate da Codema e aggiorna il
db in conseguenza. Le mail possono essere pelevate da una casella web, nel
qual caso occorre predisporre un cronjob o lette tramite un transport di
postfix.

Predisporre il transport di postfix
===================================

Questa modalità ha svariati vantaggi ma richiede una piccola preparazione sistemistica.

1. La mail indicata a Codema come mittente autorizzato (e memorizzata nel campo
   :attr:`fax.models.Device.from_user`) deve essere un alias che spedisce direttamente
   alla macchina su cui gira l'applicazione o ad un ulteriore alias. Questa operazione
   potrebbe non essere elementare con un sistema di posta 'normale', è facile con il nostro
   sistema che permette di mettere un transport personalizzato per ogni utente.

   Basta quindi chiedere al cliente di ceare un alias semplice ad una nostra casella
   (es.: faxshima1.thux.it) e creare quella casella con transport::

     smtp:[hostname-or-ip-applicazione-web]

2. La macchina su cui gira l'applicazione deve avere postfix configurato per
   dirottare la mail in arrivo su un transport specifico che invoca la script
   ``retrieve_codema_fax``

   occorre quindi:

   a. Aggiungere il dominio in mydestination di postfix, es::

        mydestination =  localhost.localdomain, localhost, thux.it

      Questo passaggio indica che le mail con destinazione il dominio thux.it
      non usciranno più da questa macchina! Per questo ho scelto thux.it e non
      thundersystems.it. Non abbiamo mail collegate a thux.it e quindi non ci
      interessa ricevere da quell'indirizzo.

   b. aggiungere in /etc/postfix/transport una riga che indichi il transport::

        thux.it faxcodema:

      e lanciare ``postmap /etc/postfix/transport``

   c. aggiungere il transport in :file:`/etc/postfix/master.cf`::

        codemafax  unix  -       n       n       -       10       pipe
            flags=DRhu user=www-data argv=/home/www/method-madrid/bin/django retrieve_codema_fax


Configurazione device
======================

Le informazioni per l'operatività concodema vengono archiviate nell'oggetto
device che nel caso di codema sarà unico per il numero di fax.

I campi configurabili per codema sono:

:from_user:  Questo è l'unico indispensabile. È l'account autorizzato da Codema
    alla spedizione. Viene impostato nel generare la mail. Attenzione che il relay
    utilizzato non deve forgiarlo diversamente (eg.: gmail)

:pop_host, pop_user, pop_password, pop_ssl: Nella configurazione che preleva
    via pop le mail pop_host, pop_user, pop_password e pop_ssl sono
    chiaramente le impostazioni necessarie per prelevare via
    pop. Personalmente credo che questa configurazione sia la meno
    conveniente e la sconsiglio (sandro). Questi paramenti vengono usati solo nel comando
    :ref:`retrieve-codema-fax <retrieve-codema-fax>` con l'opzione ``--pop``.

:smtp_host: Questa opzione è rilevante solo se si vuole loggare la comunicazione con
    codema. Il default (se il campo è vuoto) è di inviare la mail direttamente al server
    di Codema (mx.serverfax2mail.it), nel qual caso non rimanre alcun log della
    avvenuta consegna. Nel caso si sia all'interno di una rete senza accesso diretto
    all'esterno o comunque si voglia un log consiglio di impostare 'localhost'. Gli altri
    parametri vanno indicati solo se necessari per l'smtp host scelto. Senza server di
    mezzo l'operazione è normalmente più rapida (non rimbalza dagli altri server che
    nel nostro caso hanno anche l'antispam)

API
====

.. autoclass:: CodemaMsg
   :members:

.. autofunction:: get_messages

.. autofunction:: get_msg

.. _codemaonline: http://www.codemaonline.com/index.php/fax-virtuale.html

Esempi di messaggi ricevuti sono in tests/da
il modulo prevede l'opzione la cancellazione dei messaggi dopo averli letti
tramite la variabile delete passata al metodo retrieve_codema_messages

.. automodule:: fax.management.commands.retrieve_codema_fax


"""

from __future__ import unicode_literals

import re
import os
import email
import email.utils
import poplib
import filecmp
import tempfile
import datetime
from email.header import decode_header
import threading

from django.core.mail.message import EmailMessage
from django.core.files.base import ContentFile
from django.core.files import File
from django.conf import settings
from django.core.mail.backends.smtp import EmailBackend
from django.core.mail.message import EmailMessage
from django.core import mail


from jmb.fax import models
from jmb.fax import mail2fax

class CodemaMsg(mail2fax.MailMessage):

    def parse_notification(self, subject=''):
        """Parse the notification to extract info
        le notifiche di ricezione corretta recano::

           E' stato ricevuto un nuovo fax.

        le notifiche di NON spedizione::

           ERRORE: FAX NON INVIATO - Nessun circuito/canale disponibile

        """
        info = {}
        self._subject = subject
        subject = self.subject
        ## parse the subject
        # Fax inviato - 0200644640, Ulteriore prova con ID-FAX 1326 <18373>
        if re.search('formato file non corretto', subject):
            m_id = re.search("ID-FAX (\d+)", self.notification)
        else:
            m_id = re.search("ID-FAX (\d+)", subject)
        status_message = ''

        if re.search('Fax inviato', subject):
            self.status = "SENT"
            info['status'] = 4
            info['id'] = int(m_id.group(1))
            status_message = subject.replace('ID-FAX %s ' % info['id'], '')

        if re.search('PRESA IN CARICO FAX', subject):
            self.status = "SENT"
            info['status'] = 9
            m_id = re.search("PRESA IN CARICO FAX (\d+)", subject)
            info['id'] = int(m_id.group(1))
            status_message = subject.replace('ID-FAX %s ' % info['id'], '')

        elif re.search("Ricevuto fax", subject):
            self.status = "RECEIVED"
            info['status'] = 2
            m = re.search("n. registrazione (\d+)", subject)
            if m:
                status_message = "Fax ricevuto con id: %s" % m.group(1)

        elif re.search("Fax non (riuscito|inviato)|Errore fax", subject, re.I):
            self.status = "FAILED_SENT"
            info['status'] = 5
            try:
                info['id'] = int(m_id.group(1))
                status_message = subject.replace('ID-FAX %s ' % info['id'], '')
            except AttributeError:
                status_message = subject
                pass

        info['status_message'] = status_message[:199]

        # Parse the body
        for line in self.notification.split('\n'):
            try:
                key, value = line.split(': ')
            except:
                continue
            if key in ('Numero Mittente', 'Numero CND'):
                info['sender_number'] = value.strip()

            elif key == 'Nome Mittente':
                info['sender'] = value.strip()
                if info['sender'] == '':
                    info['sender'] = info['sender_number']

            elif key in ('Data/ora ricez', 'Data/ora di invio'):
                info['date_sent'] = datetime.datetime.strptime(value.strip(),
                                                               '%Y/%m/%d %H:%M:%S')
            elif key == 'Totale pagine inviate':
                info['pages'] = int(value.strip())

            elif key == 'Totale pagine':
                info['pages'] = int(value.strip())

            elif key in ('Fax ricevuto sul numero', 'Numero fax del destinatario'):
                info['recipient_number'] = value.strip()

            elif key.endswith('-id') and not 'id' in info:
                # quando Errore Fax, il subject nno viene riportato...
                # il subject sta nel corpo della mail ma senza key:
                info['id'] = value

        self.info = info

def send_via_codema(fax, content_path, pdf_cover_name=None):
    """
    Send a fax using codema service
    """
    device = models.Device.objects.get(name=fax.device)
    codema_backend = EmailBackend(
        host=device.smtp_host or 'mx.serverfax2mail.it',
        port=device.smtp_port,
        username=device.smtp_user, passsword=device.smtp_password,
        use_ssl=device.smtp_use_ssl,
    )
    from_ = device.from_user
    assert from_, "You need to set a codema `from` field"
    fax.save()

    # l'id è usato per matchare in ingresso e sapere
    # a quale record aggiornare lo stato, vedi CodemaMsg.get_or_create()
    # ID-FAX ... viene poi strippato dal parser, è l'identificativo concordato
    # con Codema che triggera l'invio di una mail di presa in carico
    subj = "ID-FAX %s - %s" % (fax.id, fax.subject, )

    body = ''
    recipient_number = re.sub('[-()\. /]', '', fax.recipient_number)
    to = ['%s@serverfax2mail.it' % recipient_number]
    msg = EmailMessage(subj, body, from_, to, connection=codema_backend)
    if pdf_cover_name:
        msg.attach_file(pdf_cover_name)
    msg.attach_file(content_path)
    # FIXME: Codema già aggiunge una cover
    msg.send(fail_silently=False)
    fax.save()
