# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import datetime

from django.utils.translation import ugettext_lazy as _
from django.db import connections
from django.db.models import Sum
from collections import OrderedDict

import admin_tools
from admin_tools.dashboard import modules, register, AppIndexDashboard

from jmb.fax.models import Fax
from jmb.core.admin import dashboard as jmb_dashboard
from jmb.core.middleware.thread_local import get_current_user
from functools import reduce

class DefaultDashboard(modules.Group):

    def __init__(self, **kwargs):

        today = datetime.date.today()
        user = get_current_user()
        children = [Pages(title=_('Report last 12 months'),)] if user.is_superuser else []
        kwargs = {
            'title': _("Faxes"),
            'display': "tabs",
            'children': children + [AllModels(), ]
        }
        super(DefaultDashboard, self).__init__(**kwargs)

class FaxDashboard(modules.Group):

    def __init__(self, **kwargs):

        today = datetime.date.today()
        kwargs = {
            'title': _("Faxes"),
            'display': "tabs",
            'children': [
                FaxModels(),
                Pages(),
                FaxOtherModels(),
                #                FaxSettings(),
                FaxPermissions(),
            ]
        }
        super(FaxDashboard, self).__init__(**kwargs)

class AllModels(modules.ModelList):

    def __init__(self, **kwargs):
        kwargs = {
            'title': _("Options"),
            'models': (
                'jmb.fax.models.Fax',
                'jmb.fax.models.Device',
                'jmb.fax.models.FaxNumber',
                'jmb.fax.models.Cover',
                'jmb.fax.models.FaxPermission'
            ),
        }
        super(AllModels, self).__init__(**kwargs)


class FaxModels(modules.ModelList):

    def __init__(self, **kwargs):
        kwargs = {
            'title': _("Models"),
            'models': ('jmb.fax.models.Fax',),
        }
        super(FaxModels, self).__init__(**kwargs)

class FaxOtherModels(modules.ModelList):

    def __init__(self, **kwargs):
        kwargs = {
            'title': _("Others"),
            'models': ('jmb.fax.models.Device', 'jmb.fax.models.FaxNumber', 'jmb.fax.models.Cover'),
        }
        super(FaxOtherModels, self).__init__(**kwargs)

class FaxSettings(modules.ModelList):

    def __init__(self, **kwargs):
        kwargs = {
            'title': _("Settings"),
            'models': ['jmb.fax.models.Settings', ],
        }
        super(FaxSettings, self).__init__(**kwargs)

class FaxPermissions(modules.ModelList):

    def __init__(self, **kwargs):
        kwargs = {
            'title': _("Permissions"),
            'models': ['jmb.fax.models.FaxPermission', ],
        }
        super(FaxPermissions, self).__init__(**kwargs)

class Faxes(modules.DashboardModule):

    def __init__(self, title, template='admin_tools/fax/faxes.html',
                 order_by=None, limit=10, qs=Fax.objects.all(), extra_filters={},
                 children=None, **kwargs):
        self.order_by = order_by
        self.template = template
        self.qs = qs
        self.extra_filters = extra_filters
        self.limit = limit
        self.custom_children = children

        kwargs = {
            'title': title,
            'template': self.template,
            'children': self.custom_children
        }
        super(Faxes, self).__init__(**kwargs)

    def init_with_context(self, context):
        request = context['request']
        if self.custom_children is None:
            qs = self.qs
            if self.extra_filters:
                qs = qs.filter(**self.extra_filters)
            if self.order_by:
                qs = qs.order_by(self.order_by)
#            if self.limit: qs = qs[:self.limit]
            self.children = qs

class Pages(modules.DashboardModule):

    def __init__(self, title=None, template='admin_tools/fax/pages.html',
                 order_by=None, limit=10, qs=Fax.objects.all(), extra_filters={},
                 children=None, **kwargs
                 ):
        start_day = (datetime.date.today()-datetime.timedelta(days=365)).replace(day=1)
        self.order_by = order_by
        self.title = title or _('Report last 12 months')
        self.template = template
        month = connections[Fax.objects.db].ops.date_trunc_sql('month', 'date_create')
        # qs = qs.filter(date_create__gte=start_day)
        # self.qs = qs.extra(select={'month': month}).values('month').annotate(
        #                              dcount=Sum('pages')).order_by('month')
        self.qs = crea_qs()

        self.extra_filters = extra_filters
        self.limit = limit
        self.custom_children = children

        kwargs = {
            'title': title,
            'template': self.template,
            'children': self.custom_children
        }
        super(Pages, self).__init__(**kwargs)

    def init_with_context(self, context):
        request = context['request']
        if self.custom_children is None:
            qs = self.qs
            if self.extra_filters:
                qs = qs.filter(**self.extra_filters)
            if self.order_by:
                qs = qs.order_by(self.order_by)
#            if self.limit: qs = qs[:self.limit]
            self.children = qs

class ListFaxes(modules.Group):

    """
    fax aggregate dashboard:
    - new fax today
    - fax to read
    """

    def __init__(self, **kwargs):

        today = datetime.date.today()
        kwargs = {
            'title': _("List Faxes"),
            'children': [
                Faxes(
                    title='Today Fax',
                    order_by='-date_create',
                    limit=None,
                    extra_filters={'date_create__gte': today},
                ),
                Faxes(
                    title='Fax to read',
                    order_by='-date_create',
                    extra_filters={'read': False},
                ),
            ]
        }
        super(ListFaxes, self).__init__(**kwargs)

class FaxModelList(modules.ModelList):

    def __init__(self, **kwargs):
        kwargs = {
            'title': _("Fax"),
            'models': ['jmb.fax.models.Fax', ],
        }
        super(FaxModelList, self).__init__(**kwargs)

class DefaultAppDashboard(AppIndexDashboard):
    #    class Media:
    #        css = ('office_thunder/css/dashboard.css',)
    #

    def __init__(self, app_title, models, **kwargs):
        AppIndexDashboard.__init__(self, app_title, models, **kwargs)
        self.columns = 2

    def init_with_context(self, context):
        request = context['request']

        # Registered dashboards
        self.children.append(
            DefaultDashboard(),
        )
class Month(object):

    def __init__(self, dic):
        for key, val in dic.items():
            setattr(self, key, val)

def crea_qs():

    # Used mainly to debug why month is a string on some systems and a datetime on others...
    start_day = (datetime.date.today()-datetime.timedelta(days=365)).replace(day=1)
    month = connections[Fax.objects.db].ops.date_trunc_sql('month', 'date_create')
    qs = Fax.objects.filter(date_create__gte=start_day, status=2)  # Received
    qs_in = qs.extra(select={'month': month}).values('month').annotate(
        count_in=Sum('pages')).order_by('month')
    qs = Fax.objects.filter(date_create__gte=start_day, status=4)  # Sent
    qs_out = qs.extra(select={'month': month}).values('month').annotate(
        count_out=Sum('pages')).order_by('month')
    months = {}

    for m in qs_in:
        months[m['month']] = {'count_in': m['count_in'] or ''}
    for m in qs_out:
        if not m['month'] in months:
            months[m['month']] = {'count_out': m['count_out'] or ''}
        else:
            months[m['month']]['count_out'] = m['count_out'] or ''
    ret = OrderedDict(sorted(list(months.items()), key=lambda t: t[0]))

    total_in = reduce(lambda x, y: x+(y or 0),
                      [months[m]['count_in'] for m in months if 'count_in' in months[m]], 0)
    total_out = reduce(lambda x, y: x+(y or 0),
                       [months[m]['count_out'] for m in months if 'count_out' in months[m]], 0)
    ret['Tot'] = {'count_in': total_in, 'count_out': total_out}
    return ret

admin_tools.dashboard.register(DefaultAppDashboard, 'jmb.fax')
#jmb_dashboard.register_group(DefaultDashboard, 'jmb.fax')
jmb_dashboard.register_group(FaxDashboard, 'jmb.fax')
