# coding: utf-8
from __future__ import unicode_literals

import os
import re
import datetime

import autocomplete_light

from django.contrib import admin, auth
from django import forms
from django.utils.translation import ugettext as _
try:
    from django.forms.utils import ErrorList
except ImportError:
    from django.forms.util import ErrorList
from django.forms.models import BaseInlineFormSet
from django.conf import settings

from jmb.fax.models import Fax, FaxNumber, Cover, Device, Document
from jmb.core.middleware.thread_local import get_request, get_current_user


class FaxFileInlineFormSet(BaseInlineFormSet):
    def clean(self):

        for form in self.forms:
            cleaned_data = form.cleaned_data
            # verifico sia validita sia non vuoto
            if form.is_valid() and len(list(cleaned_data.keys())):
                # father object
                fax = cleaned_data['fax']

                content_type = cleaned_data['file'].content_type
                if hasattr(fax, 'device') and fax.device.kpn_url:
                    # verifico i file utilizzabili
                    if content_type not in settings.FAX_ACCEPTED_CONTENT_TYPES[fax.device.type]:
                        # mozilla passa un content_type application/unknown...
                        ext = os.path.splitext(cleaned_data['file'].name)[1].strip('.')
                        if not ext in settings.FAX_ACCEPTED_EXTENSIONS[fax.device.type]:
                            msg = "Not allowed filetype/extension: %s - %s"
                            raise forms.ValidationError(msg % (content_type, ext))
        return self.cleaned_data


class DocumentInline(admin.TabularInline):
    model = Document
    fields = ('file', 'ordering',)
    extra = 0
    formset = FaxFileInlineFormSet


class CoverForm(forms.ModelForm):
    generate_cover = forms.BooleanField(required=False, label=_('Generate cover'))

    class Meta:
        model = Cover
        fields = '__all__'

    def clean(self):
        data = super(CoverForm, self).clean()
        if data['generate_cover']:
            return data
        else:
            return {}


class CoverInline(admin.StackedInline):
    model = Cover
    form = CoverForm
    max_num = 1
    fieldsets = (
        (None, {'fields': (
            ('generate_cover',),
            ('to_company',),
            ('to_person', 'from_person'),
            'pages', 'subject', 'notes'
        )}),
    )


class PhoneChoiceFields(forms.MultipleChoiceField):
    def validate(self, value):
        # I need no validation as the choice may not be present in the db
        return True


class FaxForm(forms.ModelForm):
    help_text = _("To use a number that is not present  in the address book, add  ';' at the end")
    from_number = forms.ChoiceField(required=True)
    recipient_numbers = PhoneChoiceFields(
        required=True, help_text=help_text, label=_('Recipient numbers'),
        # FIXME: Funziona correttamente ma con 2 widgets differenti
        # spostato nell'init perché questo widget non mostra i numeri quando si visita un
        # fax esistente
        # widget = autocomplete_light.MultipleChoiceWidget('RecipientAutocomplete', )
    )

    class Meta:
        model = Fax
        fields = '__all__'
    inlines = (DocumentInline, CoverInline)

    def __init__(self, *args, **kwargs):
        super(FaxForm, self).__init__(*args, **kwargs)
        ## usa come default l'unico numero disponibile, se ce n'è 1 solo...
        num_list = FaxNumber.objects.all()
        initial = None
        if len(num_list) == 1:
            initial = num_list[0].id
        self.fields['from_number'] = forms.ModelChoiceField(
            queryset=num_list, initial=initial, required=True, label=_("N. Fax"))
        if self.instance.recipient_number:
            self.fields['recipient_numbers'].choices = ([self.instance.recipient_number] * 2,)
        else:
            self.fields['recipient_numbers'].widget = autocomplete_light.MultipleChoiceWidget(
                'RecipientAutocomplete', )

    def clean(self):
        self.cleaned_data['sender_number'] = self.cleaned_data['from_number'].number
        self.cleaned_data['device'] = self.cleaned_data['from_number'].device.all()[0]

        return self.cleaned_data

    def clean_recipient_numbers(self):
        # clean the value and store the organization name to be used in recipient
        value_list = self.cleaned_data['recipient_numbers']
        if not value_list:
            raise forms.ValidationError(_('You must insert a recipient number'))

        values = []
        self._company_names = {}
        for val in value_list:

            if '[' in val:
                PATTERN = '(?P<company_name>[^()]*?) \[(?P<fax_number>[^()]+)\]'
                m = re.search(PATTERN, val)
                val = m.group('fax_number')
                self._company_names[val] = m.group('company_name')
            else:
                self._company_names[val] = ''

            values += [val]
        return values

    def save(self, *args, **kw):

        obj = super(FaxForm, self).save(commit=False)
        obj.device = self.cleaned_data['device']
        obj.sender = self.cleaned_data['from_number'].name
        obj.sender_number = self.cleaned_data['from_number'].number
        obj.save()
        return obj
