# coding: utf-8

"""
.. _hylafax:

===========
Hylafax
===========

Tutti i file che riguardano la configurazione di Hylafax stanno nella cartella
:file:/var/spool/hylafax`


Fax in ricezione
================

Alla fine di ogni ricezione viene eseguito direttamente da ``hylafax``
il comando :file:`bin/faxrcvd` che ha il seguente usage::

 Usage: $0 file devID commID error-msg [ callID-1 [ callID-2 [ ... [ callID-n ] ] ] ]

e che definisce le variabili ``FILE``, ``DEVICE``, ``COMMID``, ``MSG`` ed
ad esempio si traduce in::

 bin/faxrcvd "recvq/fax000000235.tif" "ttyS1" "000000509" "" "" ""

Per una scelta passata che non comprendo di usa solo $COMMID e si recupareno le
atre informazioni dai file di log::

  sudo bin/django hyla_notify --in $COMMID

che per esempio potrebbe essere::

  sudo bin/django hyla_notify --in C000001745

Fax inviati
================

Analogamente per fax inviati aggiungeremo la riga in :file:`bin/notify` che ha uno usage::

  Usage: $0 qfile why jobtime [nextTry]

e definisce rispettivamente le variabili ``QFILE``, ``WHY``, ``JTIME``, aggiungeremo quindi::

  QUEUE_FILE=$1
  EXIT_CODE=$2
  sudo bin/django hyla_notify --out $QUEUE_FILE -x $EXIT_CODE

che per esempio potrebbe essere::

  sudo bin/django hyla_notify --out doneq/q11 -x done


Exit Status
===========

Passato da bin/notify (solo fax inviati) può essere:

* Killed = job interrotto dall'utente con faxrm
* done = fax inviato
* failed = fax non inviato
* requeued = ovvio

Permessi
===========

django deve leggere file di log di hylafax (user uucp) e scrivere sui
file di log di django (user www-data).  La cosa più semplice è di
usare sudo come root::

  grep django /etc/sudoers
  uucp ALL= (root) NOPASSWD: /home/www/office.cogema.it/bin/django

"""

from __future__ import unicode_literals

import os
import re
try:
    from subprocess import getstatusoutput
except ImportError:
    from commands import getstatusoutput
import logging
import logging.handlers
from datetime import datetime, date

from django.conf import settings

from jmb.fax.models import Fax, Document, Device

HYLA_ROOT = '/var/spool/hylafax/'
LOG_DIR = HYLA_ROOT + 'log/'
RECV_DIR = HYLA_ROOT + 'recvq/'

PATTERN_OUT_STATUS = "(status)?(error)?"
out_status = re.compile(PATTERN_OUT_STATUS)


logger = logging.getLogger('jmb.fax')


def get_duration(filename):
    filename = "c%s" % (filename,)
    f = open(os.path.join(LOG_DIR, filename))
    duration = None

    for line in f:
        m = re.search('SEND FAX.*FROM.*docq.*sent in (?P<time>.*)\)', line)
        if m:
            duration = m.group('time')

    return duration


# FAX OK
# sdest         #### destinatario
# stty          #### device modem in uscita
# ssend         #### numero mittente
#commid
#totpages

# scommid       #### id log fax (c000000001)
# stime         #### data e ora tentativo spedizione
# sdest         #### destinatario
# ssend         #### mittente
# stty          #### device modem in uscita
# serror        #### errore del fax


class FaxOut(object):

    def __init__(self, queue, exit_status):
        """
        :arg queue: id from hylafax as doneq/q11
        """
        self.exit_status = exit_status
        qid = queue.split('/q')[1]
        logger.debug("Fax Inviato: queue id : %s" % (queue, ))
        self.fax = Fax.objects.get(job=qid)
        self.log_file_name = os.path.join(HYLA_ROOT, queue)

        self.parse_log()

    def save_and_notify(self):
        """Save the fax and notify if needed"""
        self.fax.save()
        self.fax.notify()

    def get_info_from_queue(self, log):
        info = {}
        for line in log:
            line = line.rstrip('\n')
            try:
                key = line[:line.find(':')]
                val = line[line.find(':')+1:]
                info[key] = val
            except:
                pass
        return info

    def parse_log(self):
        log = open(self.log_file_name)

        self.fax.date_last_modify = datetime.fromtimestamp(os.path.getmtime(self.log_file_name))
        self.fax.date_sent = datetime.fromtimestamp(os.path.getmtime(self.log_file_name))
        info = self.get_info_from_queue(log)

        if self.exit_status == 'killed':
            self.fax.job = ''
            self.fax.duration = ''
        else:
            # memorizziamo il file dei log
            self.fax.status_message = 'c' + info['commid']  # NOTE: serve? non è uno stato...
            if info['commid']:
                duration = get_duration(info['commid'])
            else:
                duration = None
            self.fax.duration = duration
            self.fax.sender = self.fax.device.get_first_number()
        self.fax.pages = info['totpages']

        if self.exit_status == 'done':
            self.fax.status = 4
        else:
            self.fax.status = 5
            self.fax.error = info['status'] or 'Killed'


class FaxIn(object):

    PATTERN_IN_RECV = (".{15,15}.*RECV FAX \(.*\)\: recvq\/(?P<file>.*) from " +
                       "(?P<sender>.*)\,.*, (?P<pages>.*) pages? in (?P<duration>.*)")
    PATTERN_IN_RECIPIENT = ".{15,15}.*SESSION BEGIN.* .* (?P<recipient_number>.*)"
    PATTERN_IN_DEVICE = ".{15,15}.*recvq/(?P<file>.*fax\d+.tif).*(?P<device>tty\w+?\d+?)\" "
    PATTERN_IN_SENDER = '.{15,15}.*CallID.* "(?P<sender_number>.*)" ".*" ".*" ".*"$'

    in_recv = re.compile(PATTERN_IN_RECV)
    in_recipient = re.compile(PATTERN_IN_RECIPIENT)
    in_device = re.compile(PATTERN_IN_DEVICE)
    in_sender = re.compile(PATTERN_IN_SENDER)

    def __init__(self, commid):
        """
        :arg commid: id from hylafax, e.g.: c0000010
        """
        self.fax = Fax()
        self.commid = commid
        self.log_file_name = os.path.join(LOG_DIR, commid)
        self.fax.date_create = datetime.fromtimestamp(os.path.getmtime(self.log_file_name))
        self.parse_log()

    def save_and_notify(self):
        """Save fax and notify"""
        doc = Document()
        self.fax.save()
        today = date.today().strftime('%y%m%d')
        doc.file = '%s%s_%s.pdf' % (settings.FAX_PATH_DOCUMENTS, today, self.fax.id)
        cmd = "umask 022; tiff2pdf -o %s %s%s" % (doc.file.path, RECV_DIR, self.received_tiff)
        status, output = getstatusoutput(cmd)
        self.fax.documents.add(doc)
        self.fax.save()
        self.fax.notify()

    def parse_log(self):
        log = open(self.log_file_name)

        self.fax.date_last_modify = datetime.fromtimestamp(os.path.getmtime(self.log_file_name))
        self.fax.date_sent = datetime.fromtimestamp(os.path.getmtime(self.log_file_name))
        for line in log:
            self.parse_raw_line(line)

    def parse_raw_line(self, line):

        m = self.in_recv.match(line)
        find_information = False

        if m:
            find_information = True
            self.received_tiff = m.group('file')
            self.fax.sender = m.group('sender')
            self.fax.pages = m.group('pages')
            self.fax.duration = m.group('duration')

        m = self.in_recipient.search(line)
        if m:
            self.fax.recipient_number = m.group('recipient_number')

        m = self.in_device.search(line)
        if m:
            self.fax.device = Device.objects.get(name=m.group('device'))

            # Ogni tanto ho notato che la riga in_recv non c'e e quindi ad alcuni fax
            # non viene associato il file e il numero di pagine. Quindi se non viene trovato
            # il file nella linea in_recv allora provo anche nella riga in_device
            # che sembra esserci sempre (da qua riesco solo a ricavare il nome del file e non
            # il numero di pagine)
            if not find_information:
                self.received_tiff = m.group('file')

        m = self.in_sender.search(line)
        if m:
            self.fax.sender_number = m.group('sender_number')

        self.fax.job = self.commid
        self.fax.status = 2
        self.fax.subject = 'fax ricevuto'
        self.fax.read = 0
