# coding: utf-8

from __future__ import unicode_literals

import re
import os
import email
import email.utils
import poplib
import filecmp
import tempfile
import datetime
from email.header import decode_header
import threading

from django.utils import six
from django.core.mail.message import EmailMessage
from django.core.files.base import ContentFile
from django.core.files import File
from django.conf import settings
from django.core.mail.backends.smtp import EmailBackend
from django.core.mail.message import EmailMessage
from django.core import mail


from jmb.fax import models

BACKUP_DIR = '/var/spool/codemafax'

class MailMessage(object):

    """
    Il messaggio ricevuto ha un body primario ed un allegato
    """
    fax = None
    "The Fax instance created by :meth:`get_or_create`"

    fax_content = None
    """The fax content"""

    status = None
    """Status del messaggio: RECEIVED, SENT, FAILED_RECEIVED, FAILED_SENT"""

    info = None
    """un dict con chiavi tali da poter usare ``Fax(\*\*info)``:
    sender_number, sender, date FIXME: date non esiste nel modello"""

    def __init__(self, email_msg=None, notification=None, subject='', filename=None):
        """parse the message

        :param email_msg: an ``email.message.Message`` instance retrieved from pop
        :param notification: the notification text (test)
        :param subject: the subject of the email (test)
        :param filename: a filename  that can be read w/ email.message_from_file(open...)
        """
        if filename:
            email_msg = email.message_from_file(open(filename))

        if email_msg:
            self.msg = email_msg
            if self.msg.is_multipart():
                payload_count = len(self.msg.get_payload())
                if payload_count == 2:
                    # notification + attached fax
                    self.notification_msg, self.attach_msg = self.msg.get_payload()
                elif payload_count == 1:
                    # presumbly just notification
                    self.notification_msg = self.msg.get_payload()[0]
                else:
                    raise NotImplementedError('Too many payload...')
            else:
                self.notification_msg, self.attach_msg = None, None
                self._notification = self.msg.get_payload()

        else:
            ## Mainly for test purpouses
            try:
                self._notification = notification
            except:
                assert notification is not None, "Notification must be declared if no msg"
        self.parse_notification(subject=subject)

    @property
    def subject(self):
        "Return subject in a decoded fasion"

        if not self._subject:
            self._subject = decode_subject(self.msg)
        return self._subject

    @property
    def notification(self):
        """The text of the notification received from the fax gateway"""
        try:
            return self._notification
        except AttributeError:
            return self.notification_msg.get_payload()

    def get_attach_filename(self):
        """The text of the notification received from the fax gateway"""
        return self.attach_msg.get_filename()

    @property
    def attach(self):
        """The received fax as attached pdf document"""
        return self.attach_msg.get_payload(decode=True)

    def get_or_create(self, save=True, update=True):
        """
        create and return a :class:`Fax` instance for this fax message or get from db. The
        returned fax instance is set as :attr:`CodemaMsg.fax`

        :param save: if True the fax instance is saved with new values
        :param update: if True the fax instance is updated with new values
        """
        if re.search('RECEIVED', self.status):
            fax, created = models.Fax.objects.get_or_create(
                sender_number=self.info['sender_number'], date_sent=self.info['date_sent'],
                defaults=self.info, )
            if not created:
                for key, value in self.info.items():
                    setattr(fax, key, value)

        elif re.search('SENT', self.status):
            id = self.info.pop('id')
            fax, created = models.Fax.objects.get_or_create(
                id=id,
                defaults=self.info, )
            for key, val in self.info.items():
                setattr(fax, key, val)

        self.fax = fax
        if save:
            fax.save()
        return fax, created

    def write(self, file_name=None):
        """Write the attachment to file
        :param file_name: a possible filename (default to attach_msg.get_filename())
        :return: file_name
        """
        file_name = file_name or self.attach_msg.get_filename()
        with open(file_name, 'wb') as f:
            f.write(self.attach)
        return file_name

    def save_attached_file(self):
        """Write attached file using fax instance

        :param fax: the :class:`Fax` instance describing this fax
        """
        assert self.fax, "You need to declare a fax instance before saving the attach"
        # codice di Dario!
        fax_filename = self.get_attach_filename()

        faxtemp = tempfile.NamedTemporaryFile()
        faxtemp.write(self.attach)
        faxtemp.seek(0)

        filepath = os.path.join(settings.FAX_DOCUMENT_ROOT,
                                settings.FAX_PATH_DOCUMENTS,  fax_filename)
        # verifico l'esistenza o meno del file e confronto se i file sono gli stessi
        doc = models.Document()
        doc.fax = self.fax
        if os.path.exists(filepath):
            if not filecmp.cmp(filepath, faxtemp.name):
                doc.file.save(fax_filename, ContentFile(self.attach), save=True)
            else:
                doc.file.save(fax_filename, File(open(filepath)), save=True)
        else:
            doc.file.save(fax_filename, ContentFile(self.attach), save=True)

    def notify(self, attach=True):

        assert self.fax, "You need to declare a fax instance to notify"
        self.fax.notify()

    def parse_notification(self, subject=None):
        "Derived classes should implement"
        raise NotImplementedError

    def __str__(self):
        if self.status == 'RECEIVED':
            return "<Fax %s (%s %s)>" % (self.status, self.info['sender_number'],
                                         self.info.get('date', ''))
        else:
            return "<Fax %s>" % self.status

def get_messages(host, user, pass_, key_file=None, cert_file=None, limit=1000, dump_dir=None):
    """Return all messages from host with user/pass

    :param host: the pop3 host
    :param user: the user
    :param pass_: the password
    :param key_file: a possible key (extension .pem)
    :param cert_file: a possible certificate (extension .pem)
    :param limit: e max number of messages to retrieve
    """
    if key_file:
        pop_instance = poplib.POP3_SSL(host, 995, key_file, cert_file)
    else:
        pop_instance = poplib.POP3(host)

    pop_instance.user(user)
    pop_instance.pass_(pass_)
    msg_count, mailbox_size = pop_instance.stat()
    message_list = [get_msg(pop_instance, i+1)
                    for i in range(min(msg_count, limit and int(limit)))]

    if dump_dir:
        for msg in message_list:
            save_row_message(msg)

    return message_list, pop_instance

def get_msg(pop3_instance, n):
    """Return an ``email.message.pop_instanceessage``

    :param pop3_instance: a poplib.POP3 instance
    :param n: the msg number
    :param delete_msg: true/false optional delete of messages on server
    """

    # FIXME: supposes mailbox does not change in between
    # Result is in form (response, ['line', ...], octets).

    ret = pop3_instance.retr(n)
    return email.message_from_string("\n".join(ret[1]))

def get_message_path(msg, error=False, status=''):
    """
    Return a path that holds the date of the email message

    :param msg: the message to store
    :param error: boolean if the message ha not been parsed correctly (name
        starts with error_)
    :param status: the codema_msg.status/kpn_message.status
    """
    date = None
    if isinstance(msg, six.string_types):
        date_str = None
    else:
        date_str = msg.get('date')
    if date_str:
        date_tuple = email.utils.parsedate_tz(date_str)
        if date_tuple:
            date = datetime.datetime.fromtimestamp(email.utils.mktime_tz(date_tuple))
    if date:
        fmt_date = date.strftime('%Y.%m.%d-%H:%M:%S')
    else:
        fmt_date = 'unparsed_date'

    # different names for errors
    if error:
        base = 'error_msg_%s-%s.txt'
    else:
        base = 'msg_%s-%s.txt'

    return os.path.join(BACKUP_DIR, base % (fmt_date, status or ''))

def save_row_message(msg, error=False, status=None):
    """Salva il messaggio

    :param msg:  il messaggio da salvare
    :param error: l'errore
    :param status:  non USATO
    """

    filename = get_message_path(msg, error=error)
    print("Filename: ", filename)
    f = open(filename, 'w')
    try:
        f.write(msg.as_string())
    except AttributeError:
        f.write(msg)

    f.close()
    return filename

def decode_subject(msg):
    "Return subject in a decoded fasion"

    subject = ''

    # decode returns a list; each element is a tuble (text, charset)
    for txt, charset in decode_header(msg.get('Subject')):
        if charset:
            txt = txt.decode(charset)
        subject += txt
    return subject
