# coding: utf-8
"""
.. _retrieve-codema-fax:

Retrive Codema Fax Online
=============================

Command per la ricezione di fax dal servizio codemaonline_ ,
utilizzo::

  python manage.py retrive_codema_fax
  dj retrive_codema_fax

:-d: variabile Booleana per cancellare il messaggi una volta recuperati
:-l: variabile Numero per limitare il numero di messaggi recuperare
:-v: variabile Booleana per visualizzare i print
:-b, --backup: backup messages in 
"""


from __future__ import unicode_literals

import re
import os
import sys
import email
import tempfile
import datetime
import traceback

from django.core.mail.message import EmailMessage
from django.core.management.base import BaseCommand
from django.conf import settings
from django.core import mail

from optparse import make_option

from jmb.fax.models import Fax, FaxNumber, Device
from jmb.fax import mail2fax, codema

from django.contrib.auth.models import User


class Command(BaseCommand):

    """Get all messages from configured Host
    """
    help = "Retrive Codema Fax."

    option_list = BaseCommand.option_list + (
        make_option('-d', '--delete-messages', action='store_true', default=False,
                    dest='delete_messages', help='Delete Messages'),
        make_option('-D', '--debug', action='store_true', default=False,
                    dest='debug_message', help='Debug Messages - print subject and quit'),
        make_option('-l', '--limit', dest='limit', help='Limit Messages', default=1000),
        make_option('-f', '--file', dest='filename', help='Read file in argument', default=None),
        make_option('--pop', action='store_true', help='Retrieve using pop', default=False),
        make_option('-b', '--backup', action='store_true', dest='backup',
                    help='Keep a backup of retrieved email in %s' % mail2fax.BACKUP_DIR,),
    )

    output_transaction = True

    def handle(self, **options):

        if options['pop']:
            self.retrieve_from_pop(options)
        else:
            if options['filename']:
                msg = email.message_from_file(open(options['filename']))
            else:
                msg = email.message_from_string(sys.stdin.read())
            self.parse_single_message(1, msg, options)
            if options['backup']:
                mail2fax.save_row_message(msg)

    def retrieve_from_pop(self, options):

        for device in Device.objects.filter(pop_host__isnull=False,  kpn_password__isnull=True):
            if not device.pop_host:  # ad esempio è stringa vuota...
                continue

            if options['verbosity'] == '2':
                print("Polling mbox %s " % (device.pop_user,))

            messages, pop_instance = mail2fax.get_messages(
                device.pop_host, device.pop_user, device.pop_password, limit=options['limit'],
                dump_dir=mail2fax.BACKUP_DIR if options['backup'] else None
            )

            if options['verbosity'] == '2':
                print("Number of messages: %s " % (len(messages),))
            for j, msg in enumerate(messages):
                if options['verbosity'] == '2':
                    print(j)

                self.parse_single_message(j, msg, options, pop_instance)

            pop_instance.quit()

    def parse_single_message(self, j, msg, options, pop_instance=None):

        try:
            codema_msg = codema.CodemaMsg(msg)

            fax, created = codema_msg.get_or_create(save=True, update=True)
            if options['debug_message']:
                print(codema_msg.subject)
                return
            if created:
                try:
                    codema_msg.save_attached_file()
                    codema_msg.notify()
                except Exception as e:
                    filename = mail2fax.save_row_message(msg, error=True, status=codema_msg.status)
                    send_error_to_admins(codema_msg, e, filename, show=True)
                    return
            if pop_instance and options['delete_messages']:
                pop_instance.dele(j+1)

        except Exception as e:
            error_msg = str(e) + "\n" + traceback.format_exc()
            try:
                msg = codema_msg.notification
                filename = mail2fax.save_row_message(msg, error=True, status=codema_msg.status)
            except Exception as e:  # codema object has not been created for some reason
                msg = ''
                filename = mail2fax.save_row_message(
                    msg, error=True, status=None)  # we don't have a codema_msg

            send_error_to_admins(msg, e, filename, error_msg, show=True)


def send_error_to_admins(msg, e, filename, trace=None, show=True):
    """
    Send an understandable mail msg.

    :param msg: a text message or a codema_msg
    :param e: the error raised
    :param filename: the filename whete message is saved
    :param trace: the formatted traceback of the interesting error
    """
    try:
        subject = msg.subject
    except:
        emsg = email.message_from_file(open(filename))
        subject = "%s\n%s" % (emsg.get('Subject'), emsg.get_payload())[:1000]

    body = '%s\n\n%s\n\nSubject: %s\n\n %s' % (filename, e, subject,
                                               trace or traceback.format_exc())
    mail.mail_admins("Codema retrieval problems", body)
    if show:
        print(body)
