# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import os
import re
import time
import datetime
import random
import subprocess
import logging

from django.utils.encoding import python_2_unicode_compatible
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User, Group, Permission
from django.core.files.storage import FileSystemStorage
from django.db.models import Q
from django.db import connection
from django.core.exceptions import PermissionDenied
from django.contrib.contenttypes.models import ContentType
from django.template.defaultfilters import slugify
from django.core.mail.message import EmailMessage

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jmb.core.middleware.thread_local import get_current_user
from ckeditor.fields import RichTextField
# from ckeditor.fields import RichTextField
# from jmb.core.db.models import Settings as JSettings

MAIL_INFO_BODY = _("""
Sender:   %(sender)s
Sender Number: %(sender_number)s
Recipient:   %(recipient)s
Recipient Number: %(recipient_number)s
Job ID:   %(job)s
Fax Date: %(date_sent)s
Pages:    %(pages)s
Duration: %(duration)s
""")

logger = logging.getLogger('jmb.fax')


#########################
# FAX                   #
#########################
class FaxManager(models.Manager):
    def get_queryset(self):
        user = get_current_user()
        groups = user.groups.all()

        qs = super(FaxManager, self).get_queryset()

        # FIXME : Questo if serve per lo script che chiama hylafax.
        if not user or user.is_anonymous():
            return qs

        if user.has_perm("fax.list_all_fax"):
            return qs

        q = Q(is_public=True) | \
            (Q(is_public=False) & Q(submitter=user)) | \
            (Q(is_public=False) & Q(notify_group__in=user.groups.all()))

        if user.has_perm("fax.list_all_fax_not_assigned"):
            q1 = Q(is_public=False)
            q2 = Q(notify_group__isnull=True)
            q = q | (q1 & q2)

        list_lperms = FaxPermission.objects.filter(codename__startswith='list')
        # Controllo quali numeri di fax l'utente puo' visualizzare
        #
        for perm in list_lperms:
            if user.has_perm("fax.%s" % perm.codename):
                # Recupero, a partire dai numeri di telefono associati ai permessi,
                # i device (le tty) che l'utente puo vedere.
                # Questo perche da hylafax viene passato il device ed e' quello che
                # noi andiamo a salvare nel modello dei fax.
                try:
                    allow_device = [
                        device.id for device in Device.objects.filter(numbers__number=perm.number)]
                except Exception as e:
                    return qs.none()

                qdevice = Q(device__in=allow_device)

                # Puo' vedere tutti i fax di quel numero?
                if perm.codename.find("list_all_faxes_from") >= 0:
                    q = q | qdevice

                # Puo' vedere solo quelli non assegnati?
                elif perm.codename.find("list_all_faxes_not_assigned_from") >= 0:
                    q1 = (qdevice & Q(notify_group=None))
                    q = q | q1

        return qs.filter(q)


@python_2_unicode_compatible
class Fax(DateModel, UserModel):
    status = models.IntegerField(
        default='3',
        choices=settings.FAX_STATUS_CODES,
        verbose_name=_("status"),
    )
    job = models.CharField(
        blank=True, null=True,
        max_length=15,
        verbose_name=_("job"),
        help_text=_("job reference"),
    )
    keywords = models.CharField(
        blank=True, null=True,
        max_length=100,
        verbose_name=_("keywords"),
    )
    pages = models.PositiveSmallIntegerField(
        blank=True, null=True,
        verbose_name=_("pages"),
    )
    date_required = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_("date required"),
        help_text=_("date within fax must be sent - Not implemented"),
    )
    date_sent = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_("date sent"),
    )
    ## submitter/receiver
    submitter = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        blank=True,
        null=True,
        verbose_name=_("submitter"),
    )
    ## group that can view the fax
    notify_group = models.ForeignKey(
        Group,
        blank=True,
        null=True,
        verbose_name=_("notify group"),
    )
    recipient = models.CharField(
        blank=True,
        null=True,
        max_length=100,
        verbose_name=_("recipient"),
    )
    recipient_number = models.CharField(
        max_length=16,
        verbose_name=_("recipient number"),
        help_text=_('To use a number not in the addreessbook, type ";" after the number')
    )
    sender = models.CharField(
        blank=True,
        null=True,
        max_length=100,
        verbose_name=_("sender"),
    )
    sender_number = models.CharField(
        blank=True,
        null=True,
        max_length=16,
        verbose_name=_("sender number"),
    )
    duration = models.CharField(
        blank=True,
        null=True,
        max_length=10,
        verbose_name=_("duration"),
    )
    device = models.ForeignKey(
        "Device",
        verbose_name=_("Nr. Fax/line"),
    )
    error = models.CharField(
        blank=True,
        null=True,
        max_length=150,
        verbose_name=_("error"),
    )
    subject = models.CharField(
        max_length=300,
        verbose_name=_("subject"),
    )
    read = models.IntegerField(
        null=True, blank=True,
        choices=((0, _('Not Read')), (1, _('Read'))),
        default=0,
        verbose_name=_("read"),
    )
    priority = models.PositiveSmallIntegerField(
        blank=True,
        null=True,
        default=127,
        verbose_name=_("priority"),
        choices=settings.FAX_PRIORITIES
    )
    sent_by = models.CharField(
        max_length=1,
        verbose_name=_("Sent By"),
        choices=settings.FAX_SENT_BY,
        default='W'
    )
    status_message = models.CharField(
        max_length=200, blank=True, null=True,
        verbose_name=_('Status message')
    )
    is_public = models.BooleanField(
        default=False,
    )
    objects = FaxManager()

    class Meta:
        verbose_name = _("Fax")
        verbose_name_plural = _("Fax")
        permissions = (
            ("view_fax", "Can view fax"),
            ("download_all_report", "Can view all reports"),
            ("download_all_document", "Can download all documents"),
            ("send_all_fax", "Can send all faxes"),
            ("list_all_fax", "Can list all faxes"),
            ("list_all_fax_not_assigned", "Can list all faxes not assigned"),
            ("delete_all_fax_to_send", "Can Delete ALL FAX TO SEND"),
            ("delete_all_own_fax_to_send", "Can Delete ALL OWN FAX TO SEND"),
            ("delete_all_owngroup_fax_to_send", "Can Delete ALL OWN GROUP FAX TO SEND"),
            ("delete_all_own_fax", "Can Delete ALL OWN FAX"),
            ("delete_all_owngroup_fax", "Can Delete ALL OWN GROUP FAX"),
        )
        ordering = ('-date_sent',)

    def has_cover(self):
        "Test if fax has cover w/o touching the db"
        if hasattr(self, 'cover'):
            return True
        return False

    def send(self, test=False):
        """
        Spedisce il fax eventualmente creando una cover apposita

        :param test: boolean. Don't really send, debug purpose (kpn only)
        """
        from jmb.fax.codema import send_via_codema
        from jmb.fax.kpn import send_via_kpn

        if not self.recipient_number:
            return

        self.date_sent = datetime.datetime.now()
        logger.info('Sending fax from %s to %s via %s' % (
            self.device.get_first_number(), self.recipient_number, self.device))
        logger.info('type: %s' % self.device.type)
        if self.device.type == 'kpn':
            return send_via_kpn(self, test=test)
        elif self.device.type == 'codema':
            ## FIXME Broken... but codema is no longer used (almost)
            send_via_codema(self, fax_content_path, pdf_cover_path)
        elif self.device.type == 'hyla':
            send_via_hylafax(self)

    def mark_as_read(self):
        self.read = 1

    def can_be_delete(self):
        from jmb.core.middleware.thread_local import get_current_user
        user = get_current_user()
        user_group = [g.name for g in user.groups.all()]
        list_perms = user.get_all_permissions()

        ## no one can delete faxes that have already flown away
        # that would erroneously give the ideay that the operation was ``cancelled``
        # Received, Sent, Failed
        if not self.status in (2, 4, 5):
            return False

        ## controlla permessi di delete "ALL" o "STAFF"
        if user.is_superuser or "fax.delete_fax" in list_perms:
            return True

        # Se il fax e' in stato TO SEND controllo altri tipi
        # di permessi
        if self.status == 8:
            if "fax.delete_all_fax_to_send" in list_perms:
                return True

            if "fax.delete_all_owngroup_fax_to_send" in list_perms and \
                    self.notify_group and self.notify_group.name in user_group:
                return True

            if "fax.delete_all_own_fax_to_send" in list_perms and \
                    self.submitter == user:
                return True

        # Controllo permessi di cancellazione dei proprio fax
        if "fax.delete_all_owngroup_fax" in list_perms and \
           self.notify_group and self.notify_group.name in user_group:
            return True
        if "fax.delete_all_own_fax" in list_perms and \
           self.submitter == user:
            return True

        list_lperms = FaxPermission.objects.filter(codename__startswith='delete')
        # Controllo quali numeri di fax l'utente puo' visualizzare
        # perm.model_id rappresenta il device
        #
        for perm in list_lperms:
            if user.has_perm("fax.%s" % perm.codename):
                # Se il device del fax non e' lo stesso del permesso salto a prescindere
                # tt i controlli
                try:
                    allow_device = [
                        device.name for device in Device.objects.filter(numbers__number=perm.number)]
                except Exception as e:
                    return False

                if self.device in allow_device:
                    if perm.codename.find("delete_all_fax_from") >= 0:
                        return True
                    if perm.codename.find("delete_own_fax_from") >= 0:
                        if self.submitter == user:
                            return True
                    if perm.codename.find("delete_owngroup_fax_from") >= 0:
                        if self.notify_group and self.notify_group.name in user_group:
                            return True

        return False

    # def save(self, *args, **kwargs):
    #     if not self.status == 2:
    #         self.sender_number = self.device.get_first_number()
    #     super(Fax, self).save(*args, **kwargs)

    def delete(self):
        if not self.can_be_delete():
            raise PermissionDenied
        super(Fax, self).delete()

    def notify(self):
        """
        notify all people declared in :class:`FaxNumber.send_at_received_fax`
        """
        if self.status == 2:  # Received
            recipient_list = FaxNumber.get_notifiers_in(number=self.recipient_number)
            if recipient_list:
                self.send_email(recipient_list, attach=True)

        if self.status == 4:  # Sent
            recipient_list = FaxNumber.get_notifiers_out(number=self.recipient_number)
            if settings.FAX_VIA_MAIL_NOTIFY_TO_USER:
                if not self.submitter.email in recipient_list:
                    pass

            if recipient_list:
                self.send_email(recipient_list, attach=False)

    def send_email(self, recipient_list, attach=True):
        """Invia una mail alle persone dichiarate nel :class:`FaxNumber.notify_emails`
        cui è arrivato questo fax se così configurato tramite
        :attr:`models.FaxNumber.send_at_received_fax`
        """
        data_fax = self.date_last_modify.strftime('%d/%m/%Y %H:%M:%S')
        prefix = settings.FAX_SUBJECT
        ## Subject
        if self.status == 2:
            sender = ""
            if self.sender:
                sender += "%s" % (self.sender,)
            if self.sender_number:
                sender += " - %s" % (self.sender_number,)
            subject = _('%(prefix)sReceived fax %(data_fax)s from %(sender)s') % (locals())

        elif self.status == 4:
            recipient = ""
            if self.recipient:
                recipient += "%s" % (self.recipient,)
            if self.recipient_number:
                recipient += " - %s" % (self.recipient_number,)
            subject = _('%(prefix)sSent fax %(data_fax)s to %(recipient)s') % (locals())

        from_email = 'Fax Report %s' % (settings.FAX_FROM,)
        body_message = MAIL_INFO_BODY % self.__dict__

        email = EmailMessage(subject, body_message, from_email, recipient_list)
        if attach:
            for doc in self.documents.all():
                email.attach_file(doc.file.path)
        try:
            email.send()
        except Exception as e:
            logger.error("Email not sent %s %s (%s)" % (self.sender, self.date_sent, e))

    def __str__(self):
        return '%s - %s' % (str(self.subject), self.id)


@python_2_unicode_compatible
class Document(DateModel, UserModel, OrderedModel):
    file = models.FileField(
        upload_to=settings.FAX_PATH_DOCUMENTS,
        verbose_name=_("file"),
    )
    fax = models.ForeignKey(
        Fax,
        related_name='documents'
    )

    def can_download(self, user):
        perm = 'fax.download_document_from_%s' % self.get_number()
        return user.has_perm(perm)

    def get_number(self):
        try:
            return self.fax.device.get_first_number().lstrip('+')
        except:
            return

    def __str__(self):
        return "<Document %s (%s)>" % (self.file, self.id)

    class Meta:
        verbose_name = _("Document")
        verbose_name_plural = _("Documents")
        ordering = ('ordering', 'date_create',)


@python_2_unicode_compatible
class Cover(DateModel):
    fax = models.OneToOneField(
        Fax,
        primary_key=True,
        related_name='cover'
    )
    to_company = models.CharField(
        blank=True, null=True,
        verbose_name=_("To"),
        max_length=100
    )
    from_company = models.CharField(
        blank=True, null=True,
        verbose_name=_("From"),
        max_length=100
    )
    to_person = models.CharField(
        blank=True, null=True,
        verbose_name=_("To Person"),
        max_length=100
    )
    from_person = models.CharField(
        blank=True, null=True,
        verbose_name=_("From Person"),
        max_length=100
    )
    pages = models.CharField(
        blank=True, null=True,
        verbose_name=_("Pag."),
        help_text=_("Cover Include"),
        max_length=2
    )
    subject = models.CharField(
        blank=True, null=True,
        verbose_name=_("Subject"),
        max_length=100
    )
    # notes = RichTextField
    # This field is transformed into odt by appy's function xhtml
    # so we must be sure it's correctly formatted as xhtml
    # TinyMCE does not grant this strictly so CkEditor could possibly
    # be a better solution (but not tested)
    notes = RichTextField(
        blank=True, null=True,
        verbose_name=_("Notes")
    )

    class Meta:
        verbose_name = _("Cover")
        verbose_name_plural = _("Cover")

    def get_to_person(self):
        "Return a "
        ## if field recipient is filled from autocompletion it has the form
        ## company - person

        if not self.to_person:
            try:
                company, person = self.fax.recipient.split('-')
                return person.strip()
            except:
                pass
        return self.to_person

    def get_to_company(self):
        "Return a "
        ## if field recipient is filled from autocompletion it has the form
        ## company - person

        if not self.to_person:
            try:
                company, person = self.fax.recipient.split('-')
                return company.strip()
            except:
                return self.fax.recipient.strip()

        return self.to_company

    def get_from_person(self):
        "return a nice representation of the submiter"

        if not self.from_person:
            sender = ("%s %s" % (self.fax.submitter.first_name,
                                  self.fax.submitter.last_name)).strip()
            # in case last/first name is missing
            return sender or str(self.fax.submitter)
        return self.from_person

    def save_as_pdf(self):
        """Check if a cover is needed and create it in case. Return path name
        of pdf and ps cover as a tuple, (None, None) if no cover is required.

        """
        from jmb.core.utils.appypod import AppyPodTemplate

        pdf_cover_name = "odt_file_%s_%s.pdf" % (time.time(), random.randint(0, 99))
        pdf_cover_path = os.path.join('/tmp', pdf_cover_name)

        # try to say who's sending from submitter

        context = {
            'to_company': self.get_to_company(),
            'from_company': self.from_company,
            'to_person': self.get_to_person(),
            # if no last/first name, better the username
            'from_person': self.get_from_person(),
            'fax': self.fax,
            'pages': self.pages,
            'subject': self.subject,
            'fax_date': self.fax.date_create.strftime("%d/%m/%Y %H:%M"),
            'notes': self.notes,
        }
        tpl = AppyPodTemplate(settings.FAX_COVER_TEMPLATE_NAME, context)
        return tpl.save_as(pdf_cover_path)

    def save_as(self, file_name):
        """Check if a cover is needed and create it in case. Return path name
        of pdf and ps cover as a tuple, (None, None) if no cover is required.

        :arg file_name: the filename to be created
        """
        from jmb.core.utils.appypod import AppyPodTemplate

        context = {
            'to_company': self.get_to_company(),
            'from_company': self.from_company,
            'to_person': self.get_to_person(),
            # if no last/first name, better the username
            'from_person': self.get_from_person(),
            'fax': self.fax,
            'pages': self.pages,
            'subject': self.subject,
            'fax_date': self.fax.date_create.strftime("%d/%m/%Y %H:%M"),
            'notes': self.notes,
        }
        tpl = AppyPodTemplate(settings.FAX_COVER_TEMPLATE_NAME, context)
        return tpl.save_as(file_name)

    def __str__(self):
        return "<Cover %s (%s)>" % (self.fax, self.subject)


@python_2_unicode_compatible
class Device(UserModel, DateModel, StatusModel):
    name = models.CharField(
        max_length=100, verbose_name=_("Name"),
        unique=True
    )
    "must be unique"
    description = models.CharField(
        max_length=200, blank=True, null=True,
        verbose_name=_("Description")
    )
    type = models.CharField(choices=settings.FAX_DEVICE_TYPES, max_length=12)
    from_user = models.CharField(
        max_length=50, blank=True, null=True,
        help_text=_("The email authorized to send by Codema")
    )
    kpn_password = models.CharField(
        max_length=50, blank=True, null=True
    )
    kpn_url = models.CharField(
        max_length=80, blank=True, null=True,
        help_text=_("URL used to get asyncronous notification from KPN service (MUST: port 80, 8080, or 50000+)"
                    )
    )
    "Indispensabile per codema"
    pop_host = models.CharField(
        max_length=50, blank=True, null=True,
        help_text=_('IP or hostname of host to poll via POP3')
    )
    pop_user = models.CharField(
        max_length=50, blank=True, null=True,
    )
    pop_use_ssl = models.BooleanField(
        default=False
    )
    pop_password = models.CharField(
        max_length=50, blank=True, null=True
    )
    smtp_host = models.CharField(
        max_length=50, blank=True, null=True,
        help_text=_('IP or hostname of host to use as SMTP relay')
    )
    """
    Può essere lasciato bianco nel qual caso verrà usato il server codema. È la soluzione
    più veloce ma non lascia log. Può essere invece impostato anche ``localhost``
    """
    smtp_user = models.CharField(
        max_length=50, blank=True, null=True
    )
    smtp_password = models.CharField(
        max_length=50, blank=True, null=True
    )
    smtp_port = models.IntegerField(
        default=25, blank=True, null=True
    )
    smtp_use_ssl = models.BooleanField(
        default=False
    )

    def get_first_number(self):
        try:
            return self.numbers.all()[0].number
        except:
            return

    def __str__(self):
        return "%s" % (self.name,)


class FaxNumberManager(models.Manager):
    def get_queryset(self):
        user = get_current_user()
        qs = super(FaxNumberManager, self).get_queryset()

        if user.is_superuser or user.has_perm("fax.send_all_fax"):
            return qs

        list_lperms = FaxPermission.objects.filter(codename__startswith='send')

        q = Q()

        # Controllo da quali numeri di fax l'utente puo' inviare
        for perm in list_lperms:
            if user.has_perm("fax.%s" % perm.codename):
                q = q | Q(number=perm.number)

        return qs.filter(q)


@python_2_unicode_compatible
class FaxNumber(DateModel, UserModel, StatusModel, OrderedModel):
    """
    Un numero di fax. Ad un numero sono collegati una lista di persone
    che possono ricevere una notifica quando un :class:`Fax` viene
    spedito o inviato ed un booleano che determina se detta notifica
    deve avvenire (:attr:`send_at_received_fax`,
    :attr:`send_at_sent_fax`)

    """
    device = models.ManyToManyField(
        Device,
        related_name="numbers",
        verbose_name=_("Device")
    )
    number = models.CharField(
        max_length=30,
        verbose_name=_("Number"),
        db_column="dnumber",
        unique=True
    )
    name = models.CharField(
        max_length=100,
        verbose_name=_("Name"),
        blank=True, null=True
    )
    # comma separated list
    notify_mails = models.CharField(
        max_length=200,
        blank=True, null=True,
        verbose_name=_("Notify Mails")
    )
    send_at_received_fax = models.BooleanField(default=True)
    send_at_sent_fax = models.BooleanField(default=True)
    objects = FaxNumberManager()

    def __str__(self):
        if self.name:
            return "%s - %s" % (self.number, self.name)
        return self.number

    class Meta:
        verbose_name = _("Fax Number")
        verbose_name_plural = _("Fax Numbers")
        ordering = ['ordering', ]

    def save(self, *args, **kwargs):
        if not self.id:
            content_type = ContentType.objects.get_for_model(self)
            super(FaxNumber, self).save(*args, **kwargs)
            for perm in settings.FAX_DEFAULT_PERMS:
                permission = FaxPermission()
                permission.name = perm[1] % self.number
                permission.codename = perm[0] % slugify(self.number)
                permission.content_type = content_type
                permission.number = self.number
                permission.save()
        else:
            old_number = FaxNumber.objects.get(pk=self.pk).number
            super(FaxNumber, self).save(*args, **kwargs)
            if old_number != self.number:
                for perm in settings.FAX_DEFAULT_PERMS:
                    f = FaxPermission.objects.get(codename=perm[0] % slugify(old_number))
                    f.name = perm[1] % self.number
                    f.codename = perm[0] % slugify(self.number)
                    f.save()

    def delete(self, *args, **kwargs):
        for perm in settings.FAX_DEFAULT_PERMS:
            try:
                FaxPermission.objects.get(codename=perm[0] % slugify(self.number)).delete()
            except:
                # va in eccezione se cancellati a mano
                pass
        super(FaxNumber, self).delete(*args, **kwargs)

    @classmethod
    def get_notifiers_in(cls, device=None, number=None):
        """Return a list of people that should be notified when a :class:`Fax` arrive
        on a certain device/number. At most one device or number can be passed.

        :param number: a fax number
        :param device: a fax device
        """
        # Recupero gli indirizzi mail a cui dovranno essere spedite le notifiche
        # Creo un set in modo da non avere indirizzi mail duplicati
        recipient_list = set()
        device_number = []
        if device:
            fax_number = cls.objects.filter(device__name=device)[0]
        if number:
            ## endswith: a volte manca prefix internazionale
            fax_number = cls.objects.get(number__endswith=number)

        if not (device or number):
            # va bene se esiste un solo FaxNumber
            numbers = cls.objects.all()
            if len(numbers) == 1:
                fax_numbers = numbers[0]
            else:
                assert device or number, "You must provide at least a number or a device"

        if fax_number.send_at_received_fax:
            return [m.strip() for m in fax_number.notify_mails.split(",")]
        return []

    @classmethod
    def get_notifiers_out(cls, device=None, number=None):
        """Return a list of people that should be notified when a fax arrive
        on a certain device/number. At most one device or number can be passed.

        :param number: a fax number
        :param device: a fax device
        """
        # Recupero gli indirizzi mail a cui dovranno essere spedite le notifiche
        # Creo un set in modo da non avere indirizzi mail duplicati
        recipient_list = set()
        device_number = []
        if device:
            fax_numbers = cls.objects.filter(device__name=device)
        if number:
            fax_numbers = cls.objects.filter(number=number)

        if fax_numbers:
            # it shuould be just one!
            fax_number = fax_numbers[0]

            if fax_number.send_at_sent_fax:
                return [m.strip() for m in fax_number.notify_mails.split(",")]
        return []


############################
# FAX PERMISSIONS          #
############################
class FaxPermission(Permission, UserModel, DateModel):
    number = models.CharField(
        max_length=30,
        verbose_name=_("Number"),
        db_column="dnumber",
    )


def send_via_hylafax(fax):
    """Invia il fax usando hylafax

    :param fax: l'istanza di :class:`Fax`
    :param ps_cover_name: boolean. If a cover is needed
    """
    docs = [] if not fax.has_cover() else [fax.cover.save_as_pdf().name]
    docs += [f.file.path for f in fax.documents.all()]
    logger.debug("send_via_hylafax: docs: %s" % docs)
    # Converto, per sicurezza, il file pdf in un a4 verticale
    # Infatti, quando si scanerrizza dalla stampante capita che il pdf creato
    # sia verticale ma con la page size a 842x596, invece di 596x842, e questo
    # fa si che il fax non venga spedito
    #
    recipient_number = re.sub('[-()\. /]', '', fax.recipient_number)
    info = {
        'path': '/usr/bin/sendfax',
        'start_time': 'now',
        'fax_number': recipient_number,
        'file_name': " ".join(docs or fax.cover),  # fax.file.path, #'/tmp/%s' % (ps_tmp_file,),
        'priority': fax.priority or 127,
        'device': fax.device.name + "@127.0.0.1"
    }

    cmd = "%(path)s -n -a %(start_time)s -P %(priority)s"

    # Aggiungo effettivamente la cover come cover se anche un file e'
    # stato aggiunto, altrimenti la cover verra' spedita come file
    # if fax.has_cover() and docs:
    #     cmd = cmd + " -C " + fax.cover.save_as_pdf(ps=False)
    # else:
    #     cmd = cmd + " -n"

    #    cmd = cmd + " -s a4 -B 9600 -D -d %(fax_number)s -h %(device)s %(file_name)s"
    cmd = cmd + " -s a4 -D -d %(fax_number)s -h %(device)s %(file_name)s"
    cmd = cmd % info
    logger.info(cmd)
    if settings.FAX_FAKE_SEND:
        #        import ipdb; ipdb.set_trace()

        logger.debug(cmd)
        fax.status_message = ("fake: %s" % cmd)[:200]
        fax.save()
        return

    try:
        status, output = subprocess.getstatusoutput(cmd)
    except AttributeError:
        import commands
        status, output = commands.getstatusoutput(cmd)
    ## FIXME: analizzare output. Se errore non viene propagato...
    ## se non risolve host mettere errore che non viene parsato correttamente
    logger.debug("status: %s output: %s" % (status, output))
    if status:
        raise Exception

    ## output from sendfax:
    ## request id is 879 (group id 879) for host localhost (1 file)
    fax.job = output.split()[3]
    fax.save()
